@@ -13,7 +13,8 @@ using System.Diagnostics;
 
 using Beagle.Util;
 using Beagle.Daemon;
-using Beagle.Util.Xmp;
+using FSpot;
+using FSpot.Xmp;
 using SemWeb;
 
 namespace Beagle.Filters {
@@ -50,6 +51,7 @@ namespace Beagle.Filters {
 			// use more than 100 megs memory.
 			pc.CpuLimit = 90;
 			pc.MemLimit = 100*1024*1024;
+			pc.UseLangC = true;
 
 			try {
 				pc.Start ();
