@@ -136,40 +136,21 @@ namespace Beagle.Daemon {
 
 		////////////////////////////////////////////////////////////////
 
-		// Returns the lowest matching score before the results are
-		// truncated.
-		public void DoQuery (Query               query,
-				     IQueryResult        result,
-				     ICollection         search_subset_uris, // should be internal uris
-				     QueryPartHook       query_part_hook,
-				     UriFilter           uri_filter,
-				     HitFilter           hit_filter)
+		// Returns the lists of terms in the query
+		private ArrayList AssembleQuery (Query			query,
+						 QueryPartHook		query_part_hook,
+					    	 HitFilter		hit_filter,
+					    	 out ArrayList		primary_required_part_queries,
+					    	 out ArrayList		secondary_required_part_queries,
+					    	 out LNS.BooleanQuery	primary_prohibited_part_query,
+					    	 out LNS.BooleanQuery	secondary_prohibited_part_query,
+						 out AndHitFilter	all_hit_filters)
 		{
-			if (Debug)
-				Logger.Log.Debug ("###### {0}: Starting low-level queries", IndexName);
+			primary_required_part_queries = null;
+			secondary_required_part_queries = null;
+			primary_prohibited_part_query = null;
+			secondary_prohibited_part_query = null;
 
-			Stopwatch total, a, b, c, d, e, f;
-
-			total = new Stopwatch ();
-			a = new Stopwatch ();
-			b = new Stopwatch ();
-			c = new Stopwatch ();
-			d = new Stopwatch ();
-			e = new Stopwatch ();
-			f = new Stopwatch ();
-
-			total.Start ();
-			a.Start ();
-
-			// Assemble all of the parts into a bunch of Lucene queries
-
-			ArrayList primary_required_part_queries = null;
-			ArrayList secondary_required_part_queries = null;
-
-			LNS.BooleanQuery primary_prohibited_part_query = null;
-			LNS.BooleanQuery secondary_prohibited_part_query = null;
-
-			AndHitFilter all_hit_filters;
 			all_hit_filters = new AndHitFilter ();
 			if (hit_filter != null)
 				all_hit_filters.Add (hit_filter);
@@ -209,12 +190,12 @@ namespace Beagle.Daemon {
 				case QueryPartLogic.Prohibited:
 					if (primary_prohibited_part_query == null)
 						primary_prohibited_part_query = new LNS.BooleanQuery ();
-					primary_prohibited_part_query.Add (primary_part_query, false, false);
+					primary_prohibited_part_query.Add (primary_part_query, LNS.BooleanClause.Occur.SHOULD);
 
 					if (secondary_part_query != null) {
 						if (secondary_prohibited_part_query == null)
 							secondary_prohibited_part_query = new LNS.BooleanQuery ();
-						secondary_prohibited_part_query.Add (secondary_part_query, false, false);
+						secondary_prohibited_part_query.Add (secondary_part_query, LNS.BooleanClause.Occur.SHOULD);
 					}
 
 					if (part_hit_filter != null) {
@@ -227,26 +208,16 @@ namespace Beagle.Daemon {
 				}
 			}
 
-			a.Stop ();
-			if (Debug)
-				Log.Debug ("###### {0}: Building queries took {1}", IndexName, a);
+			return term_list;
+		}
 
-			// If we have no required parts, give up.
-			if (primary_required_part_queries == null)
-				return;
-
-			b.Start ();
-			
-			//
-			// Now that we have all of these nice queries, let's execute them!
-			//
-
-			// Create the searchers that we will need.
-
-			IndexReader primary_reader;
-			LNS.IndexSearcher primary_searcher;
-			IndexReader secondary_reader = null;
-			LNS.IndexSearcher secondary_searcher = null;
+		private void BuildSearchers (out IndexReader primary_reader,
+					    out LNS.IndexSearcher primary_searcher,
+					    out IndexReader secondary_reader,
+					    out LNS.IndexSearcher secondary_searcher)
+		{
+			secondary_reader = null;
+			secondary_searcher = null;
 
 			primary_reader = LuceneCommon.GetReader (PrimaryStore);
 			primary_searcher = new LNS.IndexSearcher (primary_reader);
@@ -261,18 +232,31 @@ namespace Beagle.Daemon {
 
 			if (secondary_reader != null)
 				secondary_searcher = new LNS.IndexSearcher (secondary_reader);
+		}
 
-			b.Stop ();
-			if (Debug)
-				Log.Debug ("###### {0}: Readers/searchers built in {1}", IndexName, b);
+		private void CloseSearchers (IndexReader primary_reader,
+					    LNS.IndexSearcher primary_searcher,
+					    IndexReader secondary_reader,
+					    LNS.IndexSearcher secondary_searcher)
+		{
+			primary_searcher.Close ();
+			if (secondary_searcher != null)
+				secondary_searcher.Close ();
+			ReleaseReader (primary_reader);
+			if (secondary_reader != null)
+				ReleaseReader (secondary_reader);
+		}
 
-			// Build whitelists and blacklists for search subsets.
-			c.Start ();
-			
-			// Possibly create our whitelists from the search subset.
-
-			LuceneBitArray primary_whitelist = null;
-			LuceneBitArray secondary_whitelist = null;
+		private void CreateQueryWhitelists (ICollection		search_subset_uris,
+						    LNS.IndexSearcher	primary_searcher,
+						    LNS.IndexSearcher	secondary_searcher,
+						    LNS.BooleanQuery	primary_prohibited_part_query,
+						    LNS.BooleanQuery	secondary_prohibited_part_query,
+						    out LuceneBitArray	primary_whitelist,
+						    out LuceneBitArray	secondary_whitelist)
+		{
+			primary_whitelist = null;
+			secondary_whitelist = null;
 			
 			if (search_subset_uris != null && search_subset_uris.Count > 0) {
 				primary_whitelist = new LuceneBitArray (primary_searcher);
@@ -327,6 +311,175 @@ namespace Beagle.Daemon {
 					secondary_whitelist.AndNot (secondary_blacklist);
 				}
 			}
+		}
+
+		////////////////////////////////////////////////////////////////
+
+		public int DoCountMatchQuery (Query query, QueryPartHook query_part_hook)
+		{
+			if (Debug)
+				Logger.Log.Debug ("###### {0}: Starting low-level queries", IndexName);
+
+			Stopwatch total;
+			total = new Stopwatch ();
+			total.Start ();
+
+			ArrayList primary_required_part_queries;
+			ArrayList secondary_required_part_queries;
+
+			LNS.BooleanQuery primary_prohibited_part_query;
+			LNS.BooleanQuery secondary_prohibited_part_query;
+
+			AndHitFilter all_hit_filters;
+
+			ArrayList term_list;
+			term_list = AssembleQuery ( query,
+						    query_part_hook,
+						    null,
+						    out primary_required_part_queries,
+						    out secondary_required_part_queries,
+						    out primary_prohibited_part_query,
+						    out secondary_prohibited_part_query,
+						    out all_hit_filters);
+
+			// If we have no required parts, give up.
+			if (primary_required_part_queries == null)
+				return 0;
+
+			IndexReader primary_reader;
+			LNS.IndexSearcher primary_searcher;
+			IndexReader secondary_reader;
+			LNS.IndexSearcher secondary_searcher;
+
+			BuildSearchers (out primary_reader, out primary_searcher, out secondary_reader, out secondary_searcher);
+
+			// Build whitelists and blacklists for search subsets.
+			LuceneBitArray primary_whitelist, secondary_whitelist;
+			CreateQueryWhitelists (null,
+				primary_searcher,
+				secondary_searcher,
+				primary_prohibited_part_query,
+				secondary_prohibited_part_query,
+				out primary_whitelist,
+				out secondary_whitelist);
+
+			// Now run the low level queries against our indexes.
+			BetterBitArray primary_matches = null;
+			if (primary_required_part_queries != null) {
+
+				if (secondary_searcher != null)
+					primary_matches = DoRequiredQueries_TwoIndex (primary_searcher,
+										      secondary_searcher,
+										      primary_required_part_queries,
+										      secondary_required_part_queries,
+										      primary_whitelist,
+										      secondary_whitelist);
+				else
+					primary_matches = DoRequiredQueries (primary_searcher,
+									     primary_required_part_queries,
+									     primary_whitelist);
+
+			} 
+
+			int result = 0;
+			// FIXME: Pass the count through uri-filter and other validation checks
+			if (primary_matches != null)
+				result = primary_matches.TrueCount;
+
+			CloseSearchers (primary_reader, primary_searcher, secondary_reader, secondary_searcher);
+
+			total.Stop ();
+			if (Debug)
+				Logger.Log.Debug ("###### {0}: Total query run in {1}", IndexName, total);
+
+			return result;
+		}
+
+		////////////////////////////////////////////////////////////////
+
+		public void DoQuery (Query               query,
+				     IQueryResult        result,
+				     ICollection         search_subset_uris, // should be internal uris
+				     QueryPartHook       query_part_hook,
+				     UriFilter           uri_filter,
+				     HitFilter           hit_filter)
+		{
+			if (Debug)
+				Logger.Log.Debug ("###### {0}: Starting low-level queries", IndexName);
+
+			Stopwatch total, a, b, c, d, e, f;
+
+			total = new Stopwatch ();
+			a = new Stopwatch ();
+			b = new Stopwatch ();
+			c = new Stopwatch ();
+			d = new Stopwatch ();
+			e = new Stopwatch ();
+			f = new Stopwatch ();
+
+			total.Start ();
+			a.Start ();
+
+			ArrayList primary_required_part_queries;
+			ArrayList secondary_required_part_queries;
+
+			LNS.BooleanQuery primary_prohibited_part_query;
+			LNS.BooleanQuery secondary_prohibited_part_query;
+
+			AndHitFilter all_hit_filters;
+
+			ArrayList term_list;
+
+			// Assemble all of the parts into a bunch of Lucene queries
+
+			term_list = AssembleQuery (query,
+				query_part_hook,
+				hit_filter,
+				out primary_required_part_queries,
+				out secondary_required_part_queries,
+				out primary_prohibited_part_query,
+				out secondary_prohibited_part_query,
+				out all_hit_filters);
+
+			a.Stop ();
+			if (Debug)
+				Log.Debug ("###### {0}: Building queries took {1}", IndexName, a);
+
+			// If we have no required parts, give up.
+			if (primary_required_part_queries == null)
+				return;
+
+			b.Start ();
+			
+			//
+			// Now that we have all of these nice queries, let's execute them!
+			//
+
+			IndexReader primary_reader;
+			LNS.IndexSearcher primary_searcher;
+			IndexReader secondary_reader;
+			LNS.IndexSearcher secondary_searcher;
+
+			// Create the searchers that we will need.
+
+			BuildSearchers (out primary_reader, out primary_searcher, out secondary_reader, out secondary_searcher);
+
+			b.Stop ();
+			if (Debug)
+				Log.Debug ("###### {0}: Readers/searchers built in {1}", IndexName, b);
+
+			// Build whitelists and blacklists for search subsets.
+			c.Start ();
+
+			// Possibly create our whitelists from the search subset.
+			LuceneBitArray primary_whitelist, secondary_whitelist;
+			CreateQueryWhitelists (search_subset_uris,
+				primary_searcher,
+				secondary_searcher,
+				primary_prohibited_part_query,
+				secondary_prohibited_part_query,
+				out primary_whitelist,
+				out secondary_whitelist);
 
 			c.Stop ();
 			if (Debug)
@@ -361,8 +514,7 @@ namespace Beagle.Daemon {
 			// Only generate results if we got some matches
 			if (primary_matches != null && primary_matches.ContainsTrue ()) {
 				GenerateQueryResults (primary_reader,
-						      primary_searcher,
-						      secondary_searcher,
+						      secondary_reader,
 						      primary_matches,
 						      result,
 						      term_list,
@@ -382,14 +534,7 @@ namespace Beagle.Daemon {
 			//
 
 			f.Start ();
-			
-			primary_searcher.Close ();
-			if (secondary_searcher != null)
-				secondary_searcher.Close ();
-			ReleaseReader (primary_reader);
-			if (secondary_reader != null)
-				ReleaseReader (secondary_reader);
-
+			CloseSearchers (primary_reader, primary_searcher, secondary_reader, secondary_searcher);
 			f.Stop ();
 			
 			if (Debug)
@@ -426,7 +571,7 @@ namespace Beagle.Daemon {
 			LNS.BooleanQuery combined_query;
 			combined_query = new LNS.BooleanQuery ();
 			foreach (LNS.Query query in primary_queries)
-				combined_query.Add (query, true, false);
+				combined_query.Add (query, LNS.BooleanClause.Occur.MUST);
 
 			LuceneBitArray matches;
 			matches = new LuceneBitArray (primary_searcher, combined_query);
@@ -562,7 +707,7 @@ namespace Beagle.Daemon {
 			foreach (Term term in term_list) {
 
 				double idf;
-				idf = similarity.Ldf (reader.DocFreq (term), reader.MaxDoc ());
+				idf = similarity.Idf (reader.DocFreq (term), reader.MaxDoc ());
 
 				int hit_count;
 				hit_count = hits_by_id.Count;
@@ -598,12 +743,11 @@ namespace Beagle.Daemon {
 		//
 
 		// Two arrays we need for quickly creating lucene documents and check if they are valid
-		static string[] fields_timestamp_uri = { "Timestamp", "Uri" };
-		static string[] fields_uri = {"Uri"};
+		static FieldSelector fields_timestamp_uri = new MapFieldSelector (new string[] {"Uri", "Timestamp"});
+		static FieldSelector fields_uri = new MapFieldSelector (new string[] {"Uri"});
 
 		private static void GenerateQueryResults (IndexReader       primary_reader,
-							  LNS.IndexSearcher primary_searcher,
-							  LNS.IndexSearcher secondary_searcher,
+							  IndexReader       secondary_reader,
 							  BetterBitArray    primary_matches,
 							  IQueryResult      result,
 							  ICollection       query_term_list,
@@ -641,8 +785,7 @@ namespace Beagle.Daemon {
 
 			if (primary_matches.TrueCount > max_results)
 				final_list_of_hits = ScanRecentDocs (primary_reader,
-					primary_searcher,
-					secondary_searcher,
+					secondary_reader,
 					primary_matches,
 					hits_by_id,
 					max_results,
@@ -651,8 +794,7 @@ namespace Beagle.Daemon {
 
 			if (final_list_of_hits == null)
 				final_list_of_hits = FindRecentResults (primary_reader,
-					primary_searcher,
-					secondary_searcher,
+					secondary_reader,
 					primary_matches,
 					hits_by_id,
 					max_results,
@@ -743,8 +885,7 @@ namespace Beagle.Daemon {
 		// for all of them.
 
 		private static ArrayList ScanRecentDocs (IndexReader	    primary_reader,
-						    LNS.IndexSearcher	    primary_searcher,
-						    LNS.IndexSearcher	    secondary_searcher,
+						    IndexReader		    secondary_reader,
 						    BetterBitArray	    primary_matches,
 						    Dictionary<int, Hit>    hits_by_id,
 						    int			    max_results,
@@ -763,8 +904,8 @@ namespace Beagle.Daemon {
 
 			Term term;
 			TermDocs secondary_term_docs = null;
-			if (secondary_searcher != null)
-				secondary_term_docs = secondary_searcher.Reader.TermDocs ();
+			if (secondary_reader != null)
+				secondary_term_docs = secondary_reader.TermDocs ();
 
 			do {
 				term = enumerator.Term ();
@@ -780,13 +921,13 @@ namespace Beagle.Daemon {
 					int doc_id = docs.Doc ();
 
 					if (primary_matches.Get (doc_id)) {
-						Document doc = primary_searcher.Doc (doc_id);
+						Document doc = primary_reader.Document (doc_id);
 						// If we have a UriFilter, apply it.
 						if (uri_filter != null) {
 							Uri uri;
 							uri = GetUriFromDocument (doc);
 							if (uri_filter (uri)) {
-								Hit hit = CreateHit (doc, secondary_searcher, secondary_term_docs);
+								Hit hit = CreateHit (doc, secondary_reader, secondary_term_docs);
 								hits_by_id [doc_id] = hit;
 								// Add the result, last modified first
 								results.Add (hit);
@@ -825,8 +966,7 @@ namespace Beagle.Daemon {
 		}
 
 		private static ArrayList   FindRecentResults (IndexReader	    primary_reader,
-							      LNS.IndexSearcher primary_searcher,
-							      LNS.IndexSearcher	    secondary_searcher,
+							      IndexReader	    secondary_reader,
 							      BetterBitArray	    primary_matches,
 							      Dictionary<int, Hit>  hits_by_id,
 							      int		    max_results,
@@ -848,8 +988,8 @@ namespace Beagle.Daemon {
 			else
 				all_docs = new ArrayList (primary_matches.TrueCount);
 
-			if (secondary_searcher != null)
-				term_docs = secondary_searcher.Reader.TermDocs ();
+			if (secondary_reader != null)
+				term_docs = secondary_reader.TermDocs ();
 
 			for (int match_index = primary_matches.Count; ; match_index --) {
 				// Walk across the matches backwards, since newer
@@ -861,7 +1001,7 @@ namespace Beagle.Daemon {
 
 				count++;
 
-				doc = primary_searcher.Doc (match_index, fields_timestamp_uri);
+				doc = primary_reader.Document (match_index, fields_timestamp_uri);
 
 				// Check the timestamp --- if we have already reached our
 				// limit, we might be able to reject it immediately.
@@ -887,7 +1027,7 @@ namespace Beagle.Daemon {
 
 				// Get the actual hit now
 				// doc was created with only 2 fields, so first get the complete lucene document for primary document
-				Hit hit = CreateHit (primary_searcher.Doc (match_index), secondary_searcher, term_docs);
+				Hit hit = CreateHit (primary_reader.Document (match_index), secondary_reader, term_docs);
 				hits_by_id [match_index] = hit;
 
 				// Add the document to the appropriate data structure.
@@ -917,12 +1057,12 @@ namespace Beagle.Daemon {
 		}
 
 		private static Hit CreateHit ( Document primary_doc,
-					LNS.IndexSearcher secondary_searcher,
+					IndexReader secondary_reader,
 					TermDocs term_docs)
 		{
 			Hit hit = DocumentToHit (primary_doc);
 
-			if (secondary_searcher == null)
+			if (secondary_reader == null)
 				return hit;
 
 			// Get the stringified version of the URI
@@ -932,7 +1072,7 @@ namespace Beagle.Daemon {
 
 			// Move to the first (and only) matching term doc
 			term_docs.Next ();
-			Document secondary_doc = secondary_searcher.Doc (term_docs.Doc ());
+			Document secondary_doc = secondary_reader.Document (term_docs.Doc ());
 
 			// If we are using the secondary index, now we need to
 			// merge the properties from the secondary index
